/*
 * Copyright (C) 2004, 2005, 2006, 2007, 2016
 * Moritz Orbach <zufall@apfelboymchen.net>
 *
 * Zufall is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Zufall is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * Reads geeqie image files or files with one image per line and adds the files 
 * to a linked list.
 */
#include <stdio.h>
#include <string.h>

#include "../config.h"
#include "collection.h"
#include "zerror.h"

#define MAXLINELENGTH 1024

/*
 * reads files/directories from file and adds them to imglist
 */
static int addfile(filesearch_t *config, char const *const cpath)
{
	//printf("collection: %s\n", cpath); // debug
	FILE *fp;
	if (strcmp(cpath, "-")) { /* filenames can start with -, so cpath[0] is not good enough */
		/* fopen(3) happily opens directories in read-only mode!
		 * http://pubs.opengroup.org/onlinepubs/009695399/functions/fopen.html */
		fp = fopen(cpath, "r+");
		if (fp == NULL) {
			printerrno(cpath);
			return 1;
		}
	} else {
		fp = stdin;
	}

	char line[MAXLINELENGTH];
	unsigned int linenum = 0;
	char *rc;
	while ((rc = fgets (line, MAXLINELENGTH, fp)) != NULL) {
		int offset = 0;
		size_t ll; /* line length */
		linenum++;

		ll = strlen(line) - 1;
		if (line[ll] != '\n') {
			fprintf(stderr, "%s: line %d too long in %s\n", PACKAGE, linenum, cpath);
			/* gobble up the rest of the line. */
			rc = fgets(line, MAXLINELENGTH, fp);
			continue;
		}
		/* remove newline */
		line[ll] = '\0';
		ll--;

		switch(line[0]) {
			case '#': /* comment */
				continue;
			case '"': if (ll > 0) {
					  offset = 1;
					  if (line[ll] == '"') {
						  /* gqview-collection */
						  line[ll] = '\0';
					  } else {
						  /* ignore, likely there's more missing than just the quote */
						  fprintf(stderr, "%s: ignoring invalid line %d in %s\n", PACKAGE, linenum, cpath);
						  continue; 
					  }

				  }
				  break;
		}
		filesearch_add(config, line + offset, NULL);
	}
	if (!feof(fp)) /* NULL but not EOF */
		printerrno(cpath);

	fclose(fp);
	return 1;
}

int collection(filesearch_t *config, int pathc, char *pathv[])
{
	//printf("collections: %d\n", pathc); // debug
	int i;
	for (i = 0; i < pathc; i++)
		addfile(config, pathv[i]);
	return 1; /* all errors here should be non-fatal */
}
