/*
 * Copyright (C) 2016 Moritz Orbach <zufall@apfelboymchen.net>
 *
 * Zufall is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Zufall is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This implements a fake display server for testing purposes.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "../src/memman.h"
#include "../src/display.h"

#define NAME "fakedisplay"
#define MAXFILENAMELEN 254


typedef struct {
	int offset_x;
	int offset_y;
	int width;
	int height;
} head_t;

typedef struct display_s {
	char imgdata[MAXFILENAMELEN + 1];
	int headcount;
	head_t *heads; /* head 0 is the complete surface */
} display_t;

time_t display_sleep(display_t *display, time_t t)
{
	time_t tr;
	printf("%s: sleeping for %lu seconds\n", NAME, t);
	tr = sleep(t);
	printf("%s: sleep is over! %lu seconds remaining\n\n", NAME, t);
	return tr;
}

char const * const display_errorstr(display_t *display)
{
	return "display_errorstr()";
}

int display_renderimg(display_t *display, int head, imglib_t *imglib, char *hexcolor, int x, int y, int w, int h)
{
	if (head > display->headcount +1) {
		fprintf(stderr, "Programming error: requested head %d, but there are only %d heads\n", head, display->headcount);
		exit(3);
	}

	int offset_x = display->heads[head].offset_x;
	int offset_y = display->heads[head].offset_y;

	printf("display: render @head%d: %d,%d %dx%d (%s)\n",
			head, offset_x + x, offset_y + y, w, h, hexcolor);
	return 1;
}
int display_showimgs(display_t *display)
{
	printf("display: showimgs!\n");
	return 1;
}

/* make sure that the display server is still alive */
void display_poke(display_t *display)
{
	printf("display: %s: poked\n", NAME);
}

/* set image */
void display_setbg(display_t *display, char *filename)
{
	strncpy(display->imgdata, filename, MAXFILENAMELEN);
	display->imgdata[MAXFILENAMELEN] = '\0'; /* strncpy doesn't set \0 if the string was too large */
	printf("display: %s: background set to %s\n", NAME, display->imgdata);
}

int display_headcount(display_t *display)
{
	return display->headcount;
}

void display_headinfo(display_t *display, int head, int *x, int *y, int *w, int *h)
{
	if (head > display->headcount +1) {
		fprintf(stderr, "Programming error: requested head %d, but there are only %d heads\n", head, display->headcount);
		exit(3);
	}

	*x = display->heads[head].offset_x;
	*y = display->heads[head].offset_y;
	*w = display->heads[head].width;
	*h = display->heads[head].height;
}
display_t *display_open(char *windowname, imglib_t *imglib)
{
	display_t *display;
	display = getmem(sizeof(display_t));

	display->headcount = 2;

	display->heads = getmem(sizeof(head_t) * (display->headcount+1));
	/* first slot contains info for the complete surface */
	display->heads[0].offset_x = 0;
	display->heads[0].offset_y = 0;
	display->heads[0].width    = 1000;
	display->heads[0].height   = 1000;
	/* fake screen 1 */
	display->heads[1].offset_x = 0;
	display->heads[1].offset_y = 0;
	display->heads[1].width    = 500;
	display->heads[1].height   = 500;
	/* fake screen 1 */
	display->heads[2].offset_x = 500;
	display->heads[2].offset_y = 500;
	display->heads[2].width    = 500;
	display->heads[2].height   = 500;

	printf("display: opened\n");
	return display;
}

void display_close(display_t *display)
{
	if (display != NULL) {
		freemem(display->heads);
		freemem(display);
	}
	printf("display: closed\n");
}

